@@ -1,4 +1,3 @@
-
 import sys, os, time
 from cPickle import dump
 
@@ -9,6 +8,7 @@ from twisted.application import service
 from twisted.web import html
 
 from buildbot import interfaces, util
+from buildbot.process.properties import Properties
 
 html_tmpl = """
 <p>Changed by: <b>%(who)s</b><br />
@@ -22,6 +22,9 @@ Changed files:
 
 Comments:
 %(comments)s
+
+Properties:
+%(properties)s
 </p>
 """
 
@@ -49,29 +52,46 @@ class Change:
 
     number = None
 
-    links = []
     branch = None
+    category = None
     revision = None # used to create a source-stamp
 
-    def __init__(self, who, files, comments, isdir=0, links=[],
-                 revision=None, when=None, branch=None):
+    def __init__(self, who, files, comments, isdir=0, links=None,
+                 revision=None, when=None, branch=None, category=None,
+                 revlink='', properties={}):
         self.who = who
-        self.files = files
         self.comments = comments
         self.isdir = isdir
+        if links is None:
+            links = []
         self.links = links
         self.revision = revision
         if when is None:
             when = util.now()
         self.when = when
         self.branch = branch
+        self.category = category
+        self.revlink = revlink
+        self.properties = Properties()
+        self.properties.update(properties, "Change")
+
+        # keep a sorted list of the files, for easier display
+        self.files = files[:]
+        self.files.sort()
+
+    def __setstate__(self, dict):
+        self.__dict__ = dict
+        # Older Changes won't have a 'properties' attribute in them
+        if not hasattr(self, 'properties'):
+            self.properties = Properties()
 
     def asText(self):
         data = ""
-        data += self.getFileContents() 
+        data += self.getFileContents()
         data += "At: %s\n" % self.getTime()
         data += "Changed By: %s\n" % self.who
-        data += "Comments: %s\n\n" % self.comments
+        data += "Comments: %s" % self.comments
+        data += "Properties: \n%s\n\n" % self.getProperties()
         return data
 
     def asHTML(self):
@@ -83,19 +103,30 @@ class Change:
                 links.append('<a href="%s"><b>%s</b></a>' % (link[0], file))
             else:
                 links.append('<b>%s</b>' % file)
-        revision = ""
         if self.revision:
-            revision = "Revision: <b>%s</b><br />\n" % self.revision
+            if getattr(self, 'revlink', ""):
+                revision = 'Revision: <a href="%s"><b>%s</b></a>\n' % (
+                        self.revlink, self.revision)
+            else:
+                revision = "Revision: <b>%s</b><br />\n" % self.revision
+        else:
+            revision = ''
+
         branch = ""
         if self.branch:
             branch = "Branch: <b>%s</b><br />\n" % self.branch
 
-        kwargs = { 'who'     : html.escape(self.who),
-                   'at'      : self.getTime(),
-                   'files'   : html.UL(links) + '\n',
-                   'revision': revision,
-                   'branch'  : branch,
-                   'comments': html.PRE(self.comments) }
+        properties = []
+        for prop in self.properties.asList():
+            properties.append("%s: %s<br />" % (prop[0], prop[1]))
+
+        kwargs = { 'who'       : html.escape(self.who),
+                   'at'        : self.getTime(),
+                   'files'     : html.UL(links) + '\n',
+                   'revision'  : revision,
+                   'branch'    : branch,
+                   'comments'  : html.PRE(self.comments),
+                   'properties': html.UL(properties) + '\n' }
         return html_tmpl % kwargs
 
     def get_HTML_box(self, url):
@@ -110,8 +141,12 @@ class Change:
         pointing at the passed-in 'url'.
         """
         who = self.getShortAuthor()
+        if self.comments is None:
+            title = ""
+        else:
+            title = html.escape(self.comments)
         return '<a href="%s" title="%s">%s</a>' % (url,
-                                                   html.escape(self.comments),
+                                                   title,
                                                    html.escape(who))
 
     def getShortAuthor(self):
@@ -128,8 +163,6 @@ class Change:
 
     def getText(self):
         return [html.escape(self.who)]
-    def getColor(self):
-        return "white"
     def getLogs(self):
         return {}
 
@@ -146,6 +179,12 @@ class Change:
                 data += " %s\n" % f
         return data
         
+    def getProperties(self):
+        data = ""
+        for prop in self.properties.asList():
+            data += "  %s: %s" % (prop[0], prop[1])
+        return data
+
 class ChangeMaster(service.MultiService):
 
     """This is the master-side service which receives file change
@@ -185,6 +224,8 @@ class ChangeMaster(service.MultiService):
     debug = False
     # todo: use Maildir class to watch for changes arriving by mail
 
+    changeHorizon = 0
+
     def __init__(self):
         service.MultiService.__init__(self)
         self.changes = []
@@ -209,22 +250,25 @@ class ChangeMaster(service.MultiService):
         """Deliver a file change event. The event should be a Change object.
         This method will timestamp the object as it is received."""
         log.msg("adding change, who %s, %d files, rev=%s, branch=%s, "
-                "comments %s" % (change.who, len(change.files),
-                                 change.revision, change.branch,
-                                 change.comments))
+                "comments %s, category %s" % (change.who, len(change.files),
+                                              change.revision, change.branch,
+                                              change.comments, change.category))
         change.number = self.nextNumber
         self.nextNumber += 1
         self.changes.append(change)
         self.parent.addChange(change)
-        # TODO: call pruneChanges after a while
+        self.pruneChanges()
 
     def pruneChanges(self):
-        self.changes = self.changes[-100:] # or something
+        if self.changeHorizon and len(self.changes) > self.changeHorizon:
+            log.msg("pruning %i changes" % (len(self.changes) - self.changeHorizon))
+            self.changes = self.changes[-self.changeHorizon:]
 
-    def eventGenerator(self, branches=[]):
+    def eventGenerator(self, branches=[], categories=[]):
         for i in range(len(self.changes)-1, -1, -1):
             c = self.changes[i]
-            if not branches or c.branch in branches:
+            if (not branches or c.branch in branches) and (
+                not categories or c.category in categories):
                 yield c
 
     def getChangeNumbered(self, num):
@@ -242,7 +286,10 @@ class ChangeMaster(service.MultiService):
             return None
         offset = num - first
         log.msg(self, "offset", offset)
-        return self.changes[offset]
+        if 0 <= offset <= len(self.changes):
+            return self.changes[offset]
+        else:
+            return None
 
     def __getstate__(self):
         d = service.MultiService.__getstate__(self)
@@ -275,3 +322,8 @@ class ChangeMaster(service.MultiService):
     def stopService(self):
         self.saveYourself()
         return service.MultiService.stopService(self)
+
+class TestChangeMaster(ChangeMaster):
+    """A ChangeMaster for use in tests that does not save itself"""
+    def stopService(self):
+        return service.MultiService.stopService(self)
