@@ -1,8 +1,5 @@
 # -*- test-case-name: buildbot.test.test_changes -*-
 
-import os, os.path
-
-from twisted.application import service
 from twisted.python import log
 
 from buildbot.pbutil import NewCredPerspective
@@ -37,6 +34,10 @@ class ChangePerspective(NewCredPerspective):
                                     changedict['comments'],
                                     branch=changedict.get('branch'),
                                     revision=changedict.get('revision'),
+                                    revlink=changedict.get('revlink', ''),
+                                    category=changedict.get('category'),
+                                    when=changedict.get('when'),
+                                    properties=changedict.get('properties', {})
                                     )
             self.changemaster.addChange(change)
 
@@ -50,8 +51,8 @@ class PBChangeSource(base.ChangeSource):
         I am a ChangeSource which will accept Changes from a remote source. I
         share a TCP listening port with the buildslaves.
 
-        Both the 'buildbot sendchange' command and the
-        contrib/svn_buildbot.py tool know how to send changes to me.
+        The 'buildbot sendchange' command, the contrib/svn_buildbot.py tool,
+        and the contrib/bzr_buildbot.py tool know how to send changes to me.
 
         @type prefix: string (or None)
         @param prefix: if set, I will ignore any filenames that do not start
@@ -108,4 +109,3 @@ class PBChangeSource(base.ChangeSource):
 
     def getPerspective(self):
         return ChangePerspective(self.parent, self.prefix)
-
