@@ -13,6 +13,7 @@ from buildbot.changes import base
 from buildbot.changes.changes import Change
 
 import xml.dom.minidom
+import urllib
 
 def _assert(condition, msg):
     if condition:
@@ -47,7 +48,7 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
     compare_attrs = ["svnurl", "split_file_function",
                      "svnuser", "svnpasswd",
                      "pollinterval", "histmax",
-                     "svnbin"]
+                     "svnbin", "category"]
 
     parent = None # filled in when we're added
     last_change = None
@@ -57,7 +58,7 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
     def __init__(self, svnurl, split_file=None,
                  svnuser=None, svnpasswd=None,
                  pollinterval=10*60, histmax=100,
-                 svnbin='svn'):
+                 svnbin='svn', revlinktmpl='', category=None):
         """
         @type  svnurl: string
         @param svnurl: the SVN URL that describes the repository and
@@ -163,6 +164,18 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         @param svnbin:       path to svn binary, defaults to just 'svn'. Use
                              this if your subversion command lives in an
                              unusual location.
+        
+        @type  revlinktmpl:  string
+        @param revlinktmpl:  A format string to use for hyperlinks to revision
+                             information. For example, setting this to
+                             "http://reposerver/websvn/revision.php?rev=%s"
+                             would create suitable links on the build pages
+                             to information in websvn on each revision.
+
+        @type  category:     string
+        @param category:     A single category associated with the changes that
+                             could be used by schedulers watch for branches of a
+                             certain name AND category.
         """
 
         if svnurl.endswith("/"):
@@ -172,12 +185,15 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         self.svnuser = svnuser
         self.svnpasswd = svnpasswd
 
+        self.revlinktmpl = revlinktmpl
+
         self.svnbin = svnbin
         self.pollinterval = pollinterval
         self.histmax = histmax
         self._prefix = None
         self.overrun_counter = 0
         self.loop = LoopingCall(self.checksvn)
+        self.category = category
 
     def split_file(self, path):
         # use getattr() to avoid turning this function into a bound method,
@@ -365,8 +381,11 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
 
 
     def _get_text(self, element, tag_name):
-        child_nodes = element.getElementsByTagName(tag_name)[0].childNodes
-        text = "".join([t.data for t in child_nodes])
+        try:
+            child_nodes = element.getElementsByTagName(tag_name)[0].childNodes
+            text = "".join([t.data for t in child_nodes])
+        except:
+            text = "<unknown>"
         return text
 
     def _transform_path(self, path):
@@ -386,6 +405,13 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
         for el in new_logentries:
             branch_files = [] # get oldest change first
             revision = str(el.getAttribute("revision"))
+
+            revlink=''
+
+            if self.revlinktmpl:
+                if revision:
+                    revlink = self.revlinktmpl % urllib.quote_plus(revision)
+
             dbgMsg("Adding change revision %s" % (revision,))
             # TODO: the rest of buildbot may not be ready for unicode 'who'
             # values
@@ -436,7 +462,9 @@ class SVNPoller(base.ChangeSource, util.ComparableMixin):
                                files=files,
                                comments=comments,
                                revision=revision,
-                               branch=branch)
+                               branch=branch,
+                               revlink=revlink,
+                               category=self.category)
                     changes.append(c)
 
         return changes
