@@ -4,7 +4,7 @@
 Define the interfaces that are implemented by various buildbot classes.
 """
 
-from zope.interface import Interface
+from zope.interface import Interface, Attribute
 
 # exceptions that can be raised while trying to start a build
 class NoSlaveError(Exception):
@@ -13,6 +13,12 @@ class BuilderInUseError(Exception):
     pass
 class BuildSlaveTooOldError(Exception):
     pass
+class LatentBuildSlaveFailedToSubstantiate(Exception):
+    pass
+
+# other exceptions
+class BuildbotNotRunningError(Exception):
+    pass
 
 class IChangeSource(Interface):
     """Object which feeds Change objects to the changemaster. When files or
@@ -39,7 +45,7 @@ class IScheduler(Interface):
     """I watch for Changes in the source tree and decide when to trigger
     Builds. I create BuildSet objects and submit them to the BuildMaster. I
     am a service, and the BuildMaster is always my parent.
-    
+
     @ivar properties: properties to be applied to all builds started by this
     scheduler
     @type properties: L<buildbot.process.properties.Properties>
@@ -76,6 +82,16 @@ class IUpstreamScheduler(Interface):
         """Return a list of strings indicating the Builders that this
         Scheduler might feed."""
 
+class IDownstreamScheduler(Interface):
+    """This marks an IScheduler to be listening to other schedulers.
+    On reconfigs, these might get notified to check if their upstream
+    scheduler are stil the same."""
+
+    def checkUpstreamScheduler():
+        """Check if the upstream scheduler is still alive, and if not,
+        get a new upstream object from the master."""
+
+
 class ISourceStamp(Interface):
     """
     @cvar branch: branch from which source was drawn
@@ -289,6 +305,10 @@ class IBuildRequestStatus(Interface):
         be called once for each of these Builds, both old and new."""
     def unsubscribe(observer):
         """Unregister the callable that was registered with subscribe()."""
+    def getSubmitTime():
+        """Return the time when this request was submitted"""
+    def setSubmitTime(t):
+        """Sets the time when this request was submitted"""
 
 
 class ISlaveStatus(Interface):
@@ -322,6 +342,9 @@ class IBuilderStatus(Interface):
     def getName():
         """Return the name of this Builder (a string)."""
 
+    def getCategory():
+        """Return the category of this builder (a string)."""
+
     def getState():
         # TODO: this isn't nearly as meaningful as it used to be
         """Return a tuple (state, builds) for this Builder. 'state' is the
@@ -420,7 +443,7 @@ class IBuilderStatus(Interface):
         delivered."""
 
 class IEventSource(Interface):
-    def eventGenerator(branches=[]):
+    def eventGenerator(branches=[], categories=[]):
         """This function creates a generator which will yield all of this
         object's status events, starting with the most recent and progressing
         backwards in time. These events provide the IStatusEvent interface.
@@ -431,6 +454,11 @@ class IEventSource(Interface):
         return events that are associated with these branches. If the list is
         empty, events for all branches should be returned (i.e. an empty list
         means 'accept all' rather than 'accept none').
+
+        @param categories: a list of category names.  The generator
+        should only return events that are categorized within the
+        given category.  If the list is empty, events for all
+        categories should be returned.
         """
 
 class IBuildStatus(Interface):
@@ -440,7 +468,7 @@ class IBuildStatus(Interface):
     def getBuilder():
         """
         Return the BuilderStatus that owns this build.
-        
+
         @rtype: implementor of L{IBuilderStatus}
         """
 
@@ -534,15 +562,10 @@ class IBuildStatus(Interface):
         available, the caller should join them together with spaces before
         presenting them to the user."""
 
-    def getColor():
-        """Returns a single string with the color that should be used to
-        display the build. 'green', 'orange', or 'red' are the most likely
-        ones."""
-
     def getResults():
         """Return a constant describing the results of the build: one of the
-        constants in buildbot.status.builder: SUCCESS, WARNINGS, or
-        FAILURE."""
+        constants in buildbot.status.builder: SUCCESS, WARNINGS,
+        FAILURE, SKIPPED or EXCEPTION."""
 
     def getLogs():
         """Return a list of logs that describe the build as a whole. Some
@@ -659,11 +682,6 @@ class IBuildStepStatus(Interface):
         available, the caller should join them together with spaces before
         presenting them to the user."""
 
-    def getColor():
-        """Returns a single string with the color that should be used to
-        display this step. 'green', 'orange', 'red' and 'yellow' are the
-        most likely ones."""
-
     def getResults():
         """Return a tuple describing the results of the step: (result,
         strings). 'result' is one of the constants in
@@ -702,10 +720,6 @@ class IStatusEvent(Interface):
         available, the caller should join them together with spaces before
         presenting them to the user."""
 
-    def getColor():
-        """Returns a single string with the color that should be used to
-        display this event. 'red' and 'yellow' are the most likely ones."""
-
 
 LOG_CHANNEL_STDOUT = 0
 LOG_CHANNEL_STDERR = 1
@@ -870,6 +884,19 @@ class IStatusReceiver(Interface):
         @type buildset: implementor of L{IBuildSetStatus}
         """
 
+    def requestSubmitted(request):
+        """A new BuildRequest has been submitted to the buildmaster.
+
+        @type request: implementor of L{IBuildRequestStatus}
+        """
+
+    def requestCancelled(builder, request):
+        """A BuildRequest has been cancelled on the given Builder.
+
+        @type builder: L{buildbot.status.builder.BuilderStatus}
+        @type request: implementor of L{IBuildRequestStatus}
+        """
+
     def builderAdded(builderName, builder):
         """
         A new Builder has just been added. This method may return an
@@ -919,6 +946,18 @@ class IStatusReceiver(Interface):
         logStarted/logFinished messages, it will also receive stepETAUpdate
         messages about every updateInterval seconds."""
 
+    def stepTextChanged(build, step, text):
+        """The text for a step has been updated.
+
+        This is called when calling setText() on the step status, and
+        hands in the text list."""
+
+    def stepText2Changed(build, step, text2):
+        """The text2 for a step has been updated.
+
+        This is called when calling setText2() on the step status, and
+        hands in text2 list."""
+
     def stepETAUpdate(build, step, ETA, expectations):
         """This is a periodic update on the progress this Step has made
         towards completion. It gets an ETA (in seconds from the present) of
@@ -1004,12 +1043,13 @@ class IBuilderControl(Interface):
         the build that is currently in progress: once the build finishes,
         there is nothing to control anymore."""
 
-    def ping(timeout=30):
+    def ping():
         """Attempt to contact the slave and see if it is still alive. This
         returns a Deferred which fires with either True (the slave is still
-        alive) or False (the slave did not respond). As a side effect, adds
-        an event to this builder's column in the waterfall display
-        containing the results of the ping."""
+        alive) or False (the slave did not respond). As a side effect, adds an
+        event to this builder's column in the waterfall display containing the
+        results of the ping. Note that this may not fail for a long time, it is
+        implemented in terms of the timeout on the underlying TCP connection."""
         # TODO: this ought to live in ISlaveControl, maybe with disconnect()
         # or something. However the event that is emitted is most useful in
         # the Builder column, so it kinda fits here too.
@@ -1068,3 +1108,32 @@ class ILogObserver(Interface):
 class IBuildSlave(Interface):
     # this is a marker interface for the BuildSlave class
     pass
+
+class ILatentBuildSlave(IBuildSlave):
+    """A build slave that is not always running, but can run when requested.
+    """
+    substantiated = Attribute('Substantiated',
+                              'Whether the latent build slave is currently '
+                              'substantiated with a real instance.')
+
+    def substantiate():
+        """Request that the slave substantiate with a real instance.
+
+        Returns a deferred that will callback when a real instance has
+        attached."""
+
+    # there is an insubstantiate too, but that is not used externally ATM.
+
+    def buildStarted(sb):
+        """Inform the latent build slave that a build has started.
+
+        ``sb`` is a LatentSlaveBuilder as defined in buildslave.py.  The sb
+        is the one for whom the build started.
+        """
+
+    def buildFinished(sb):
+        """Inform the latent build slave that a build has finished.
+
+        ``sb`` is a LatentSlaveBuilder as defined in buildslave.py.  The sb
+        is the one for whom the build finished.
+        """
