@@ -11,22 +11,26 @@ import warnings
 
 from zope.interface import implements
 from twisted.python import log, components
+from twisted.python.failure import Failure
 from twisted.internet import defer, reactor
 from twisted.spread import pb
 from twisted.cred import portal, checkers
 from twisted.application import service, strports
 from twisted.persisted import styles
 
+import buildbot
 # sibling imports
-from buildbot.util import now
+from buildbot.util import now, safeTranslate
 from buildbot.pbutil import NewCredPerspective
 from buildbot.process.builder import Builder, IDLE
 from buildbot.process.base import BuildRequest
 from buildbot.status.builder import Status
-from buildbot.changes.changes import Change, ChangeMaster
+from buildbot.changes.changes import Change, ChangeMaster, TestChangeMaster
 from buildbot.sourcestamp import SourceStamp
 from buildbot.buildslave import BuildSlave
-from buildbot import interfaces
+from buildbot import interfaces, locks
+from buildbot.process.properties import Properties
+from buildbot.config import BuilderConfig
 
 ########################################
 
@@ -60,6 +64,14 @@ class BotMaster(service.MultiService):
         # self.locks holds the real Lock instances
         self.locks = {}
 
+        # self.mergeRequests is the callable override for merging build
+        # requests
+        self.mergeRequests = None
+
+        # self.prioritizeBuilders is the callable override for builder order
+        # traversal
+        self.prioritizeBuilders = None
+
     # these four are convenience functions for testing
 
     def waitUntilBuilderAttached(self, name):
@@ -187,8 +199,41 @@ class BotMaster(service.MultiService):
         return defer.DeferredList(dl)
 
     def maybeStartAllBuilds(self):
-        for b in self.builders.values():
-            b.maybeStartBuild()
+        builders = self.builders.values()
+        if self.prioritizeBuilders is not None:
+            try:
+                builders = self.prioritizeBuilders(self.parent, builders)
+            except:
+                log.msg("Exception prioritizing builders")
+                log.err(Failure())
+                return
+        else:
+            def _sortfunc(b1, b2):
+                t1 = b1.getOldestRequestTime()
+                t2 = b2.getOldestRequestTime()
+                # If t1 or t2 is None, then there are no build requests,
+                # so sort it at the end
+                if t1 is None:
+                    return 1
+                if t2 is None:
+                    return -1
+                return cmp(t1, t2)
+            builders.sort(_sortfunc)
+        try:
+            for b in builders:
+                b.maybeStartBuild()
+        except:
+            log.msg("Exception starting builds")
+            log.err(Failure())
+
+    def shouldMergeRequests(self, builder, req1, req2):
+        """Determine whether two BuildRequests should be merged for
+        the given builder.
+
+        """
+        if self.mergeRequests is not None:
+            return self.mergeRequests(builder, req1, req2)
+        return req1.canBeMergedWith(req2)
 
     def getPerspective(self, slavename):
         return self.slaves[slavename]
@@ -209,6 +254,7 @@ class BotMaster(service.MultiService):
         @param lockid: a locks.MasterLock or locks.SlaveLock instance
         @return: a locks.RealMasterLock or locks.RealSlaveLock instance
         """
+        assert isinstance(lockid, (locks.MasterLock, locks.SlaveLock))
         if not lockid in self.locks:
             self.locks[lockid] = lockid.lockClass(lockid)
         # if the master.cfg file has changed maxCount= on the lock, the next
@@ -227,11 +273,13 @@ class DebugPerspective(NewCredPerspective):
     def detached(self, mind):
         pass
 
-    def perspective_requestBuild(self, buildername, reason, branch, revision):
+    def perspective_requestBuild(self, buildername, reason, branch, revision, properties={}):
         c = interfaces.IControl(self.master)
         bc = c.getBuilder(buildername)
         ss = SourceStamp(branch, revision)
-        br = BuildRequest(reason, ss, buildername)
+        bpr = Properties()
+        bpr.update(properties, "remote requestBuild")
+        br = BuildRequest(reason, ss, builderName=buildername, properties=bpr)
         bc.requestBuild(br)
 
     def perspective_pingBuilder(self, buildername):
@@ -273,18 +321,12 @@ class DebugPerspective(NewCredPerspective):
     def perspective_print(self, msg):
         print "debug", msg
 
-class Dispatcher(styles.Versioned):
+class Dispatcher:
     implements(portal.IRealm)
-    persistenceVersion = 2
 
     def __init__(self):
         self.names = {}
 
-    def upgradeToVersion1(self):
-        self.master = self.botmaster.parent
-    def upgradeToVersion2(self):
-        self.names = {}
-
     def register(self, name, afactory):
         self.names[name] = afactory
     def unregister(self, name):
@@ -330,15 +372,15 @@ class Dispatcher(styles.Versioned):
 #   UNIXServer(ResourcePublisher(self.site))
 
 
-class BuildMaster(service.MultiService, styles.Versioned):
+class BuildMaster(service.MultiService):
     debug = 0
-    persistenceVersion = 3
     manhole = None
     debugPassword = None
     projectName = "(unspecified)"
     projectURL = None
     buildbotURL = None
     change_svc = None
+    properties = Properties()
 
     def __init__(self, basedir, configFileName="master.cfg"):
         service.MultiService.__init__(self)
@@ -374,27 +416,10 @@ class BuildMaster(service.MultiService, styles.Versioned):
         # this ChangeMaster is a dummy, only used by tests. In the real
         # buildmaster, where the BuildMaster instance is activated
         # (startService is called) by twistd, this attribute is overwritten.
-        self.useChanges(ChangeMaster())
+        self.useChanges(TestChangeMaster())
 
         self.readConfig = False
 
-    def upgradeToVersion1(self):
-        self.dispatcher = self.slaveFactory.root.portal.realm
-
-    def upgradeToVersion2(self): # post-0.4.3
-        self.webServer = self.webTCPPort
-        del self.webTCPPort
-        self.webDistribServer = self.webUNIXPort
-        del self.webUNIXPort
-        self.configFileName = "master.cfg"
-
-    def upgradeToVersion3(self):
-        # post 0.6.3, solely to deal with the 0.6.3 breakage. Starting with
-        # 0.6.5 I intend to do away with .tap files altogether
-        self.services = []
-        self.namedServices = {}
-        del self.change_svc
-
     def startService(self):
         service.MultiService.startService(self)
         self.loadChanges() # must be done before loading the config file
@@ -447,6 +472,7 @@ class BuildMaster(service.MultiService, styles.Versioned):
         if not configFile:
             configFile = os.path.join(self.basedir, self.configFileName)
 
+        log.msg("Creating BuildMaster -- buildbot.version: %s" % buildbot.version)
         log.msg("loading configuration from %s" % configFile)
         configFile = os.path.expanduser(configFile)
 
@@ -491,9 +517,13 @@ class BuildMaster(service.MultiService, styles.Versioned):
 
         known_keys = ("bots", "slaves",
                       "sources", "change_source",
-                      "schedulers", "builders",
-                      "slavePortnum", "debugPassword", "manhole",
-                      "status", "projectName", "projectURL", "buildbotURL",
+                      "schedulers", "builders", "mergeRequests",
+                      "slavePortnum", "debugPassword", "logCompressionLimit",
+                      "manhole", "status", "projectName", "projectURL",
+                      "buildbotURL", "properties", "prioritizeBuilders",
+                      "eventHorizon", "buildCacheSize", "logHorizon", "buildHorizon",
+                      "changeHorizon", "logMaxSize", "logMaxTailSize",
+                      "logCompressionMethod",
                       )
         for k in config.keys():
             if k not in known_keys:
@@ -503,13 +533,6 @@ class BuildMaster(service.MultiService, styles.Versioned):
             # required
             schedulers = config['schedulers']
             builders = config['builders']
-            for k in builders:
-                if k['name'].startswith("_"):
-                    errmsg = ("builder names must not start with an "
-                              "underscore: " + k['name'])
-                    log.err(errmsg)
-                    raise ValueError(errmsg)
-
             slavePortnum = config['slavePortnum']
             #slaves = config['slaves']
             #change_source = config['change_source']
@@ -521,6 +544,35 @@ class BuildMaster(service.MultiService, styles.Versioned):
             projectName = config.get('projectName')
             projectURL = config.get('projectURL')
             buildbotURL = config.get('buildbotURL')
+            properties = config.get('properties', {})
+            buildCacheSize = config.get('buildCacheSize', None)
+            eventHorizon = config.get('eventHorizon', None)
+            logHorizon = config.get('logHorizon', None)
+            buildHorizon = config.get('buildHorizon', None)
+            logCompressionLimit = config.get('logCompressionLimit', 4*1024)
+            if logCompressionLimit is not None and not \
+                    isinstance(logCompressionLimit, int):
+                raise ValueError("logCompressionLimit needs to be bool or int")
+            logCompressionMethod = config.get('logCompressionMethod', "bz2")
+            if logCompressionMethod not in ('bz2', 'gz'):
+                raise ValueError("logCompressionMethod needs to be 'bz2', or 'gz'")
+            logMaxSize = config.get('logMaxSize')
+            if logMaxSize is not None and not \
+                    isinstance(logMaxSize, int):
+                raise ValueError("logMaxSize needs to be None or int")
+            logMaxTailSize = config.get('logMaxTailSize')
+            if logMaxTailSize is not None and not \
+                    isinstance(logMaxTailSize, int):
+                raise ValueError("logMaxTailSize needs to be None or int")
+            mergeRequests = config.get('mergeRequests')
+            if mergeRequests is not None and not callable(mergeRequests):
+                raise ValueError("mergeRequests must be a callable")
+            prioritizeBuilders = config.get('prioritizeBuilders')
+            if prioritizeBuilders is not None and not callable(prioritizeBuilders):
+                raise ValueError("prioritizeBuilders must be callable")
+            changeHorizon = config.get("changeHorizon")
+            if changeHorizon is not None and not isinstance(changeHorizon, int):
+                raise ValueError("changeHorizon needs to be an int")
 
         except KeyError, e:
             log.msg("config dictionary is missing a required parameter")
@@ -547,6 +599,9 @@ class BuildMaster(service.MultiService, styles.Versioned):
         #if "sources" in config:
         #    raise KeyError("c['sources'] is no longer accepted")
 
+        if changeHorizon is not None:
+            self.change_svc.changeHorizon = changeHorizon
+
         change_source = config.get('change_source', [])
         if isinstance(change_source, (list, tuple)):
             change_sources = change_source
@@ -562,9 +617,10 @@ class BuildMaster(service.MultiService, styles.Versioned):
 
         # do some validation first
         for s in slaves:
-            assert isinstance(s, BuildSlave)
+            assert interfaces.IBuildSlave.providedBy(s)
             if s.slavename in ("debug", "change", "status"):
-                raise KeyError, "reserved name '%s' used for a bot" % s.slavename
+                raise KeyError(
+                    "reserved name '%s' used for a bot" % s.slavename)
         if config.has_key('interlocks'):
             raise KeyError("c['interlocks'] is no longer accepted")
 
@@ -584,10 +640,19 @@ class BuildMaster(service.MultiService, styles.Versioned):
         slavenames = [s.slavename for s in slaves]
         buildernames = []
         dirnames = []
+
+        # convert builders from objects to config dictionaries
+        builders_dicts = []
+        for b in builders:
+            if isinstance(b, BuilderConfig):
+                builders_dicts.append(b.getConfigDict())
+            elif type(b) is dict:
+                builders_dicts.append(b)
+            else:
+                raise ValueError("builder %s is not a BuilderConfig object (or a dict)" % b)
+        builders = builders_dicts
+
         for b in builders:
-            if type(b) is tuple:
-                raise ValueError("builder %s must be defined with a dict, "
-                                 "not a tuple" % b[0])
             if b.has_key('slavename') and b['slavename'] not in slavenames:
                 raise ValueError("builder %s uses undefined slave %s" \
                                  % (b['name'], b['slavename']))
@@ -599,6 +664,19 @@ class BuildMaster(service.MultiService, styles.Versioned):
                 raise ValueError("duplicate builder name %s"
                                  % b['name'])
             buildernames.append(b['name'])
+
+            # sanity check name (BuilderConfig does this too)
+            if b['name'].startswith("_"):
+                errmsg = ("builder names must not start with an "
+                          "underscore: " + b['name'])
+                log.err(errmsg)
+                raise ValueError(errmsg)
+
+            # Fix the dictionnary with default values, in case this wasn't
+            # specified with a BuilderConfig object (which sets the same defaults)
+            b.setdefault('builddir', safeTranslate(b['name']))
+            b.setdefault('slavebuilddir', b['builddir'])
+
             if b['builddir'] in dirnames:
                 raise ValueError("builder %s reuses builddir %s"
                                  % (b['name'], b['builddir']))
@@ -629,28 +707,35 @@ class BuildMaster(service.MultiService, styles.Versioned):
 
         # assert that all locks used by the Builds and their Steps are
         # uniquely named.
-        locks = {}
+        lock_dict = {}
         for b in builders:
             for l in b.get('locks', []):
-                if locks.has_key(l.name):
-                    if locks[l.name] is not l:
+                if isinstance(l, locks.LockAccess): # User specified access to the lock
+                    l = l.lockid
+                if lock_dict.has_key(l.name):
+                    if lock_dict[l.name] is not l:
                         raise ValueError("Two different locks (%s and %s) "
                                          "share the name %s"
-                                         % (l, locks[l.name], l.name))
+                                         % (l, lock_dict[l.name], l.name))
                 else:
-                    locks[l.name] = l
+                    lock_dict[l.name] = l
             # TODO: this will break with any BuildFactory that doesn't use a
             # .steps list, but I think the verification step is more
             # important.
             for s in b['factory'].steps:
                 for l in s[1].get('locks', []):
-                    if locks.has_key(l.name):
-                        if locks[l.name] is not l:
+                    if isinstance(l, locks.LockAccess): # User specified access to the lock
+                        l = l.lockid
+                    if lock_dict.has_key(l.name):
+                        if lock_dict[l.name] is not l:
                             raise ValueError("Two different locks (%s and %s)"
                                              " share the name %s"
-                                             % (l, locks[l.name], l.name))
+                                             % (l, lock_dict[l.name], l.name))
                     else:
-                        locks[l.name] = l
+                        lock_dict[l.name] = l
+
+        if not isinstance(properties, dict):
+            raise ValueError("c['properties'] must be a dictionary")
 
         # slavePortnum supposed to be a strports specification
         if type(slavePortnum) is int:
@@ -667,6 +752,32 @@ class BuildMaster(service.MultiService, styles.Versioned):
         self.projectURL = projectURL
         self.buildbotURL = buildbotURL
 
+        self.properties = Properties()
+        self.properties.update(properties, self.configFileName)
+
+        self.status.logCompressionLimit = logCompressionLimit
+        self.status.logCompressionMethod = logCompressionMethod
+        self.status.logMaxSize = logMaxSize
+        self.status.logMaxTailSize = logMaxTailSize
+        # Update any of our existing builders with the current log parameters.
+        # This is required so that the new value is picked up after a
+        # reconfig.
+        for builder in self.botmaster.builders.values():
+            builder.builder_status.setLogCompressionLimit(logCompressionLimit)
+            builder.builder_status.setLogCompressionMethod(logCompressionMethod)
+            builder.builder_status.setLogMaxSize(logMaxSize)
+            builder.builder_status.setLogMaxTailSize(logMaxTailSize)
+
+        if mergeRequests is not None:
+            self.botmaster.mergeRequests = mergeRequests
+        if prioritizeBuilders is not None:
+            self.botmaster.prioritizeBuilders = prioritizeBuilders
+
+        self.buildCacheSize = buildCacheSize
+        self.eventHorizon = eventHorizon
+        self.logHorizon = logHorizon
+        self.buildHorizon = buildHorizon
+
         # self.slaves: Disconnect any that were attached and removed from the
         # list. Update self.checker with the new list of passwords, including
         # debug/change/status.
@@ -744,6 +855,8 @@ class BuildMaster(service.MultiService, styles.Versioned):
         # shut down any that were removed, start any that were added
         deleted_sources = [s for s in self.change_svc if s not in sources]
         added_sources = [s for s in sources if s not in self.change_svc]
+        log.msg("adding %d new changesources, removing %d" %
+                (len(added_sources), len(deleted_sources)))
         dl = [self.change_svc.removeSource(s) for s in deleted_sources]
         def addNewOnes(res):
             [self.change_svc.addSource(s) for s in added_sources]
@@ -762,10 +875,21 @@ class BuildMaster(service.MultiService, styles.Versioned):
         added = [s for s in newschedulers if s not in oldschedulers]
         dl = [defer.maybeDeferred(s.disownServiceParent) for s in removed]
         def addNewOnes(res):
+            log.msg("adding %d new schedulers, removed %d" % 
+                    (len(added), len(dl)))
             for s in added:
                 s.setServiceParent(self)
         d = defer.DeferredList(dl, fireOnOneErrback=1)
         d.addCallback(addNewOnes)
+        if removed or added:
+            # notify Downstream schedulers to potentially pick up
+            # new schedulers now that we have removed and added some
+            def updateDownstreams(res):
+                log.msg("notifying downstream schedulers of changes")
+                for s in newschedulers:
+                    if interfaces.IDownstreamScheduler.providedBy(s):
+                        s.checkUpstreamScheduler()
+            d.addCallback(updateDownstreams)
         return d
 
     def loadConfig_Builders(self, newBuilderData):
@@ -790,7 +914,7 @@ class BuildMaster(service.MultiService, styles.Versioned):
         # everything in newList is either unchanged, changed, or new
         for name, data in newList.items():
             old = self.botmaster.builders.get(name)
-            basedir = data['builddir'] # used on both master and slave
+            basedir = data['builddir']
             #name, slave, builddir, factory = data
             if not old: # new
                 # category added after 0.6.2
@@ -826,6 +950,12 @@ class BuildMaster(service.MultiService, styles.Versioned):
                 log.msg("builder %s is unchanged" % name)
                 pass
 
+        # regardless of whether anything changed, get each builder status
+        # to update its config
+        for builder in allBuilders.values():
+            builder.builder_status.reconfigFromBuildmaster(self)
+
+        # and then tell the botmaster if anything's changed
         if somethingChanged:
             sortedAllBuilders = [allBuilders[name] for name in newBuilderNames]
             d = self.botmaster.setBuilders(sortedAllBuilders)
@@ -896,4 +1026,3 @@ components.registerAdapter(Control, BuildMaster, interfaces.IControl)
 
 # so anybody who can get a handle on the BuildMaster can cause a build with:
 #  IControl(master).getBuilder("full-2.3").requestBuild(buildrequest)
-
