@@ -8,33 +8,113 @@ from twisted.python import usage, util, runtime
 
 from buildbot.interfaces import BuildbotNotRunningError
 
-# this is mostly just a front-end for mktap, twistd, and kill(1), but in the
-# future it will also provide an interface to some developer tools that talk
-# directly to a remote buildmaster (like 'try' and a status client)
-
 # the create/start/stop commands should all be run as the same user,
 # preferably a separate 'buildbot' account.
 
-class MakerBase(usage.Options):
+# Note that the terms 'options' and 'config' are used intechangeably here - in
+# fact, they are intercanged several times.  Caveat legator.
+
+class OptionsWithOptionsFile(usage.Options):
+    # subclasses should set this to a list-of-lists in order to source the
+    # .buildbot/options file.  
+    # buildbotOptions = [ [ 'optfile-name', 'option-name' ], .. ]
+    buildbotOptions = None
+
+    def __init__(self, *args):
+        # for options in self.buildbotOptions, optParameters, and the options
+        # file, change the default in optParameters *before* calling through
+        # to the parent constructor
+
+        if self.buildbotOptions:
+            optfile = loadOptionsFile()
+            for optfile_name, option_name in self.buildbotOptions:
+                for i in range(len(self.optParameters)):
+                    if self.optParameters[i][0] == option_name and optfile_name in optfile:
+                        self.optParameters[i][2] = optfile[optfile_name]
+        usage.Options.__init__(self, *args)
+
+def loadOptionsFile(filename="options", here=None, home=None):
+    """Find the .buildbot/FILENAME file. Crawl from the current directory up
+    towards the root, and also look in ~/.buildbot . The first directory
+    that's owned by the user and has the file we're looking for wins. Windows
+    skips the owned-by-user test.
+    
+    @rtype:  dict
+    @return: a dictionary of names defined in the options file. If no options
+             file was found, return an empty dict.
+    """
+
+    if here is None:
+        here = os.getcwd()
+    here = os.path.abspath(here)
+
+    if home is None:
+        if runtime.platformType == 'win32':
+            home = os.path.join(os.environ['APPDATA'], "buildbot")
+        else:
+            home = os.path.expanduser("~/.buildbot")
+
+    searchpath = []
+    toomany = 20
+    while True:
+        searchpath.append(os.path.join(here, ".buildbot"))
+        next = os.path.dirname(here)
+        if next == here:
+            break # we've hit the root
+        here = next
+        toomany -= 1 # just in case
+        if toomany == 0:
+            raise ValueError("Hey, I seem to have wandered up into the "
+                             "infinite glories of the heavens. Oops.")
+    searchpath.append(home)
+
+    localDict = {}
+
+    for d in searchpath:
+        if os.path.isdir(d):
+            if runtime.platformType != 'win32':
+                if os.stat(d)[stat.ST_UID] != os.getuid():
+                    print "skipping %s because you don't own it" % d
+                    continue # security, skip other people's directories
+            optfile = os.path.join(d, filename)
+            if os.path.exists(optfile):
+                try:
+                    f = open(optfile, "r")
+                    options = f.read()
+                    exec options in localDict
+                except:
+                    print "error while reading %s" % optfile
+                    raise
+                break
+
+    for k in localDict.keys():
+        if k.startswith("__"):
+            del localDict[k]
+    return localDict
+
+class MakerBase(OptionsWithOptionsFile):
     optFlags = [
         ['help', 'h', "Display this message"],
         ["quiet", "q", "Do not emit the commands being run"],
         ]
 
-    #["basedir", "d", None, "Base directory for the buildmaster"],
+    longdesc = """
+    Operates upon the specified <basedir> (or the current directory, if not
+    specified).
+    """
+
     opt_h = usage.Options.opt_help
 
     def parseArgs(self, *args):
         if len(args) > 0:
             self['basedir'] = args[0]
         else:
-            self['basedir'] = None
+            # Use the current directory if no basedir was specified.
+            self['basedir'] = os.getcwd()
         if len(args) > 1:
             raise usage.UsageError("I wasn't expecting so many arguments")
 
     def postOptions(self):
-        if self['basedir'] is None:
-            raise usage.UsageError("<basedir> parameter is required")
         self['basedir'] = os.path.abspath(self['basedir'])
 
 makefile_sample = """# -*- makefile -*-
@@ -95,6 +175,10 @@ class Maker:
             f.write("Please put a description of this build host here\n")
             f.close()
             created = True
+        access_uri = os.path.join(path, "access_uri")
+        if not os.path.exists(access_uri):
+            if not self.quiet:
+                print "Not creating info/access_uri - add it if you wish"
         if created and not self.quiet:
             print "Please edit the files in %s appropriately." % path
 
@@ -156,7 +240,7 @@ class Maker:
         f.close()
         os.chmod(target, 0600)
 
-    def public_html(self, index_html, buildbot_css, robots_txt):
+    def public_html(self, files):
         webdir = os.path.join(self.basedir, "public_html")
         if os.path.exists(webdir):
             if not self.quiet:
@@ -166,20 +250,11 @@ class Maker:
             os.mkdir(webdir)
         if not self.quiet:
             print "populating public_html/"
-        target = os.path.join(webdir, "index.html")
-        f = open(target, "wt")
-        f.write(open(index_html, "rt").read())
-        f.close()
-
-        target = os.path.join(webdir, "buildbot.css")
-        f = open(target, "wt")
-        f.write(open(buildbot_css, "rt").read())
-        f.close()
-
-        target = os.path.join(webdir, "robots.txt")
-        f = open(target, "wt")
-        f.write(open(robots_txt, "rt").read())
-        f.close()
+        for target, source in files.iteritems():
+            target = os.path.join(webdir, target)
+            f = open(target, "wt")
+            f.write(open(source, "rt").read())
+            f.close()
 
     def populate_if_missing(self, target, source, overwrite=False):
         new_contents = open(source, "rt").read()
@@ -202,18 +277,15 @@ class Maker:
                 print "populating %s" % target
             open(target, "wt").write(new_contents)
 
-    def upgrade_public_html(self, index_html, buildbot_css, robots_txt):
+    def upgrade_public_html(self, files):
         webdir = os.path.join(self.basedir, "public_html")
         if not os.path.exists(webdir):
             if not self.quiet:
                 print "populating public_html/"
             os.mkdir(webdir)
-        self.populate_if_missing(os.path.join(webdir, "index.html"),
-                                 index_html)
-        self.populate_if_missing(os.path.join(webdir, "buildbot.css"),
-                                 buildbot_css)
-        self.populate_if_missing(os.path.join(webdir, "robots.txt"),
-                                 robots_txt)
+        for target, source in files.iteritems():
+            self.populate_if_missing(os.path.join(webdir, target),
+                                 source)
 
     def check_master_cfg(self):
         from buildbot.master import BuildMaster
@@ -273,7 +345,7 @@ class UpgradeMasterOptions(MakerBase):
         ]
 
     def getSynopsis(self):
-        return "Usage:    buildbot upgrade-master [options] <basedir>"
+        return "Usage:    buildbot upgrade-master [options] [<basedir>]"
 
     longdesc = """
     This command takes an existing buildmaster working directory and
@@ -296,12 +368,14 @@ def upgradeMaster(config):
     m = Maker(config)
     # TODO: check Makefile
     # TODO: check TAC file
-    # check web files: index.html, classic.css, robots.txt
+    # check web files: index.html, default.css, robots.txt
     webdir = os.path.join(basedir, "public_html")
-    m.upgrade_public_html(util.sibpath(__file__, "../status/web/index.html"),
-                          util.sibpath(__file__, "../status/web/classic.css"),
-                          util.sibpath(__file__, "../status/web/robots.txt"),
-                          )
+    m.upgrade_public_html({
+          'index.html' : util.sibpath(__file__, "../status/web/index.html"),
+          'bg_gradient.jpg' : util.sibpath(__file__, "../status/web/bg_gradient.jpg"),
+          'buildbot.css' : util.sibpath(__file__, "../status/web/default.css"),
+          'robots.txt' : util.sibpath(__file__, "../status/web/robots.txt"),
+      })
     m.populate_if_missing(os.path.join(basedir, "master.cfg.sample"),
                           util.sibpath(__file__, "sample.cfg"),
                           overwrite=True)
@@ -325,7 +399,7 @@ class MasterOptions(MakerBase):
          "limit the number of kept old twisted log files"],
         ]
     def getSynopsis(self):
-        return "Usage:    buildbot create-master [options] <basedir>"
+        return "Usage:    buildbot create-master [options] [<basedir>]"
 
     longdesc = """
     This command creates a buildmaster working directory and buildbot.tac
@@ -378,10 +452,12 @@ def createMaster(config):
     contents = masterTAC % config
     m.makeTAC(contents)
     m.sampleconfig(util.sibpath(__file__, "sample.cfg"))
-    m.public_html(util.sibpath(__file__, "../status/web/index.html"),
-                  util.sibpath(__file__, "../status/web/classic.css"),
-                  util.sibpath(__file__, "../status/web/robots.txt"),
-                  )
+    m.public_html({
+          'index.html' : util.sibpath(__file__, "../status/web/index.html"),
+          'bg_gradient.jpg' : util.sibpath(__file__, "../status/web/bg_gradient.jpg"),
+          'buildbot.css' : util.sibpath(__file__, "../status/web/default.css"),
+          'robots.txt' : util.sibpath(__file__, "../status/web/robots.txt"),
+      })
     m.makefile()
 
     if not m.quiet: print "buildmaster configured in %s" % m.basedir
@@ -518,7 +594,12 @@ def stop(config, signame="TERM", wait=False):
     pid = int(f.read().strip())
     signum = getattr(signal, "SIG"+signame)
     timer = 0
-    os.kill(pid, signum)
+    try:
+        os.kill(pid, signum)
+    except OSError, e:
+        if e.errno != 3:
+            raise
+
     if not wait:
         if not quiet:
             print "sent SIG%s to process" % signame
@@ -549,82 +630,23 @@ def restart(config):
     start(config)
 
 
-def loadOptions(filename="options", here=None, home=None):
-    """Find the .buildbot/FILENAME file. Crawl from the current directory up
-    towards the root, and also look in ~/.buildbot . The first directory
-    that's owned by the user and has the file we're looking for wins. Windows
-    skips the owned-by-user test.
-    
-    @rtype:  dict
-    @return: a dictionary of names defined in the options file. If no options
-             file was found, return an empty dict.
-    """
-
-    if here is None:
-        here = os.getcwd()
-    here = os.path.abspath(here)
-
-    if home is None:
-        if runtime.platformType == 'win32':
-            home = os.path.join(os.environ['APPDATA'], "buildbot")
-        else:
-            home = os.path.expanduser("~/.buildbot")
-
-    searchpath = []
-    toomany = 20
-    while True:
-        searchpath.append(os.path.join(here, ".buildbot"))
-        next = os.path.dirname(here)
-        if next == here:
-            break # we've hit the root
-        here = next
-        toomany -= 1 # just in case
-        if toomany == 0:
-            raise ValueError("Hey, I seem to have wandered up into the "
-                             "infinite glories of the heavens. Oops.")
-    searchpath.append(home)
-
-    localDict = {}
-
-    for d in searchpath:
-        if os.path.isdir(d):
-            if runtime.platformType != 'win32':
-                if os.stat(d)[stat.ST_UID] != os.getuid():
-                    print "skipping %s because you don't own it" % d
-                    continue # security, skip other people's directories
-            optfile = os.path.join(d, filename)
-            if os.path.exists(optfile):
-                try:
-                    f = open(optfile, "r")
-                    options = f.read()
-                    exec options in localDict
-                except:
-                    print "error while reading %s" % optfile
-                    raise
-                break
-
-    for k in localDict.keys():
-        if k.startswith("__"):
-            del localDict[k]
-    return localDict
-
 class StartOptions(MakerBase):
     optFlags = [
         ['quiet', 'q', "Don't display startup log messages"],
         ]
     def getSynopsis(self):
-        return "Usage:    buildbot start <basedir>"
+        return "Usage:    buildbot start [<basedir>]"
 
 class StopOptions(MakerBase):
     def getSynopsis(self):
-        return "Usage:    buildbot stop <basedir>"
+        return "Usage:    buildbot stop [<basedir>]"
 
 class ReconfigOptions(MakerBase):
     optFlags = [
         ['quiet', 'q', "Don't display log messages about reconfiguration"],
         ]
     def getSynopsis(self):
-        return "Usage:    buildbot reconfig <basedir>"
+        return "Usage:    buildbot reconfig [<basedir>]"
 
 
 
@@ -633,9 +655,9 @@ class RestartOptions(MakerBase):
         ['quiet', 'q', "Don't display startup log messages"],
         ]
     def getSynopsis(self):
-        return "Usage:    buildbot restart <basedir>"
+        return "Usage:    buildbot restart [<basedir>]"
 
-class DebugClientOptions(usage.Options):
+class DebugClientOptions(OptionsWithOptionsFile):
     optFlags = [
         ['help', 'h', "Display this message"],
         ]
@@ -643,6 +665,11 @@ class DebugClientOptions(usage.Options):
         ["master", "m", None,
          "Location of the buildmaster's slaveport (host:port)"],
         ["passwd", "p", None, "Debug password to use"],
+        ["myoption", "O", "DEF", "My Option!"],
+        ]
+    buildbotOptions = [
+        [ 'debugMaster', 'passwd' ],
+        [ 'master', 'master' ],
         ]
 
     def parseArgs(self, *args):
@@ -653,20 +680,19 @@ class DebugClientOptions(usage.Options):
         if len(args) > 2:
             raise usage.UsageError("I wasn't expecting so many arguments")
 
+    def postOptions(self):
+        print self['myoption']
+        sys.exit(1)
+
 def debugclient(config):
     from buildbot.clients import debug
-    opts = loadOptions()
 
     master = config.get('master')
-    if not master:
-        master = opts.get('master')
     if master is None:
         raise usage.UsageError("master must be specified: on the command "
                                "line or in ~/.buildbot/options")
 
     passwd = config.get('passwd')
-    if not passwd:
-        passwd = opts.get('debugPassword')
     if passwd is None:
         raise usage.UsageError("passwd must be specified: on the command "
                                "line or in ~/.buildbot/options")
@@ -674,7 +700,7 @@ def debugclient(config):
     d = debug.DebugWidget(master, passwd)
     d.run()
 
-class StatusClientOptions(usage.Options):
+class StatusClientOptions(OptionsWithOptionsFile):
     optFlags = [
         ['help', 'h', "Display this message"],
         ]
@@ -682,6 +708,9 @@ class StatusClientOptions(usage.Options):
         ["master", "m", None,
          "Location of the buildmaster's status port (host:port)"],
         ]
+    buildbotOptions = [
+        [ 'masterstatus', 'master' ],
+    ]
 
     def parseArgs(self, *args):
         if len(args) > 0:
@@ -691,10 +720,7 @@ class StatusClientOptions(usage.Options):
 
 def statuslog(config):
     from buildbot.clients import base
-    opts = loadOptions()
     master = config.get('master')
-    if not master:
-        master = opts.get('masterstatus')
     if master is None:
         raise usage.UsageError("master must be specified: on the command "
                                "line or in ~/.buildbot/options")
@@ -703,17 +729,18 @@ def statuslog(config):
 
 def statusgui(config):
     from buildbot.clients import gtkPanes
-    opts = loadOptions()
     master = config.get('master')
-    if not master:
-        master = opts.get('masterstatus')
     if master is None:
         raise usage.UsageError("master must be specified: on the command "
                                "line or in ~/.buildbot/options")
     c = gtkPanes.GtkClient(master)
     c.run()
 
-class SendChangeOptions(usage.Options):
+class SendChangeOptions(OptionsWithOptionsFile):
+    def __init__(self):
+        OptionsWithOptionsFile.__init__(self)
+        self['properties'] = {}
+
     optParameters = [
         ("master", "m", None,
          "Location of the buildmaster's PBListener (host:port)"),
@@ -723,15 +750,28 @@ class SendChangeOptions(usage.Options):
         ("revision", "r", None, "Revision specifier (string)"),
         ("revision_number", "n", None, "Revision specifier (integer)"),
         ("revision_file", None, None, "Filename containing revision spec"),
+        ("property", "p", None,
+         "A property for the change, in the format: name:value"),
         ("comments", "m", None, "log message"),
         ("logfile", "F", None,
          "Read the log messages from this file (- for stdin)"),
         ("when", "w", None, "timestamp to use as the change time"),
         ]
+
+    buildbotOptions = [
+        [ 'master', 'master' ],
+        [ 'username', 'username' ],
+        [ 'branch', 'branch' ],
+        [ 'category', 'category' ],
+    ]
+
     def getSynopsis(self):
         return "Usage:    buildbot sendchange [options] filenames.."
     def parseArgs(self, *args):
         self['files'] = args
+    def opt_property(self, property):
+        name,value = property.split(':')
+        self['properties'][name] = value
 
 
 def sendchange(config, runReactor=False):
@@ -739,12 +779,12 @@ def sendchange(config, runReactor=False):
     connection will be drpoped as soon as the Change has been sent."""
     from buildbot.clients.sendchange import Sender
 
-    opts = loadOptions()
-    user = config.get('username', opts.get('username'))
-    master = config.get('master', opts.get('master'))
-    branch = config.get('branch', opts.get('branch'))
-    category = config.get('category', opts.get('category'))
+    user = config.get('username')
+    master = config.get('master')
+    branch = config.get('branch')
+    category = config.get('category')
     revision = config.get('revision')
+    properties = config.get('properties', {})
     if config.get('when'):
         when = float(config.get('when'))
     else:
@@ -771,7 +811,8 @@ def sendchange(config, runReactor=False):
     assert master, "you must provide the master location"
 
     s = Sender(master, user)
-    d = s.send(branch, revision, comments, files, category=category, when=when)
+    d = s.send(branch, revision, comments, files, category=category, when=when,
+               properties=properties)
     if runReactor:
         d.addCallbacks(s.printSuccess, s.printFailure)
         d.addBoth(s.stop)
@@ -779,7 +820,7 @@ def sendchange(config, runReactor=False):
     return d
 
 
-class ForceOptions(usage.Options):
+class ForceOptions(OptionsWithOptionsFile):
     optParameters = [
         ["builder", None, None, "which Builder to start"],
         ["branch", None, None, "which branch to build"],
@@ -799,7 +840,7 @@ class ForceOptions(usage.Options):
             self['reason'] = " ".join(args)
 
 
-class TryOptions(usage.Options):
+class TryOptions(OptionsWithOptionsFile):
     optParameters = [
         ["connect", "c", None,
          "how to reach the buildmaster, either 'ssh' or 'pb'"],
@@ -832,6 +873,12 @@ class TryOptions(usage.Options):
          "Run the trial build on this Builder. Can be used multiple times."],
         ["properties", None, None,
          "A set of properties made available in the build environment, format:prop=value,propb=valueb..."],
+
+        ["try-topfile", None, None,
+         "Name of a file at the top of the tree, used to find the top. Only needed for SVN and CVS."],
+        ["try-topdir", None, None,
+         "Path to the top of the working copy. Only needed for SVN and CVS."],
+
         ]
 
     optFlags = [
@@ -839,8 +886,26 @@ class TryOptions(usage.Options):
         ["dryrun", 'n', "Gather info, but don't actually submit."],
         ]
 
+    # here it is, the definitive, quirky mapping of .buildbot/options names to
+    # command-line options.  Design by committee, anyone?
+    buildbotOptions = [
+        [ 'try_connect', 'connect' ],
+        #[ 'try_builders', 'builders' ], <-- handled in postOptions
+        [ 'try_vc', 'vc' ],
+        [ 'try_branch', 'branch' ],
+        [ 'try_topdir', 'try-topdir' ],
+        [ 'try_topfile', 'try-topfile' ],
+        [ 'try_host', 'tryhost' ],
+        [ 'try_username', 'username' ],
+        [ 'try_dir', 'trydir' ],
+        [ 'try_password', 'passwd' ],
+        [ 'try_master', 'master' ],
+        #[ 'try_wait', 'wait' ], <-- handled in postOptions
+        [ 'masterstatus', 'master' ],
+    ]
+
     def __init__(self):
-        super(TryOptions, self).__init__()
+        OptionsWithOptionsFile.__init__(self)
         self['builders'] = []
         self['properties'] = {}
 
@@ -863,12 +928,19 @@ class TryOptions(usage.Options):
     def getSynopsis(self):
         return "Usage:    buildbot try [options]"
 
+    def postOptions(self):
+        opts = loadOptionsFile()
+        if not self['builders']:
+            self['builders'] = opts.get('try_builders', [])
+        if opts.get('try_wait', False):
+            self['wait'] = True
+
 def doTry(config):
     from buildbot.scripts import tryclient
     t = tryclient.Try(config)
     t.run()
 
-class TryServerOptions(usage.Options):
+class TryServerOptions(OptionsWithOptionsFile):
     optParameters = [
         ["jobdir", None, None, "the jobdir (maildir) for submitting jobs"],
         ]
@@ -891,7 +963,7 @@ def doTryServer(config):
     os.rename(tmpfile, newfile)
 
 
-class CheckConfigOptions(usage.Options):
+class CheckConfigOptions(OptionsWithOptionsFile):
     optFlags = [
         ['quiet', 'q', "Don't display error messages or tracebacks"],
     ]
