@@ -70,6 +70,9 @@ class RemoteBuilder(pb.Referenceable):
     def remote_getName(self):
         return self.b.getName()
 
+    def remote_getCategory(self):
+        return self.b.getCategory()
+
     def remote_getState(self):
         state, builds = self.b.getState()
         return (state,
@@ -175,9 +178,6 @@ class RemoteBuild(pb.Referenceable):
     def remote_getText(self):
         return self.b.getText()
 
-    def remote_getColor(self):
-        return self.b.getColor()
-
     def remote_getResults(self):
         return self.b.getResults()
 
@@ -267,9 +267,6 @@ class RemoteBuildStep(pb.Referenceable):
     def remote_getText(self):
         return self.s.getText()
 
-    def remote_getColor(self):
-        return self.s.getColor()
-
     def remote_getResults(self):
         return self.s.getResults()
 
@@ -300,8 +297,6 @@ class RemoteEvent:
         return self.s.getTimes()
     def remote_getText(self):
         return self.s.getText()
-    def remote_getColor(self):
-        return self.s.getColor()
 
 components.registerAdapter(RemoteEvent,
                            interfaces.IStatusEvent, IRemote)
@@ -430,6 +425,10 @@ class StatusClientPerspective(base.StatusReceiverPerspective):
         s = self.status.getSlave(name)
         return IRemote(s)
 
+    def perspective_ping(self):
+        """Ping method to allow pb clients to validate their connections."""
+        return "pong"
+
     # IStatusReceiver methods, invoked if we've subscribed
 
     # mode >= builder
