@@ -16,12 +16,15 @@ from buildbot.status.web.base import HtmlResource, Box, \
 from buildbot.status.web.feeds import Rss20StatusResource, \
      Atom10StatusResource
 from buildbot.status.web.waterfall import WaterfallStatusResource
-from buildbot.status.web.grid import GridStatusResource
+from buildbot.status.web.console import ConsoleStatusResource
+from buildbot.status.web.grid import GridStatusResource, TransposedGridStatusResource
 from buildbot.status.web.changes import ChangesResource
 from buildbot.status.web.builder import BuildersResource
+from buildbot.status.web.buildstatus import BuildStatusStatusResource 
 from buildbot.status.web.slaves import BuildSlavesResource
 from buildbot.status.web.xmlrpc import XMLRPCServer
 from buildbot.status.web.about import AboutBuildbot
+from buildbot.status.web.auth import IAuth, AuthFailResource
 
 # this class contains the status services (WebStatus and the older Waterfall)
 # which can be put in c['status']. It also contains some of the resources
@@ -70,7 +73,7 @@ def getLastNBuilds(status, numbuilds, builders=[], branches=[]):
 
 
 # /one_line_per_build
-#  accepts builder=, branch=, numbuilds=
+#  accepts builder=, branch=, numbuilds=, reload=
 class OneLinePerBuild(HtmlResource, OneLineMixin):
     """This shows one line per build, combining all builders together. Useful
     query arguments:
@@ -78,6 +81,7 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
     numbuilds=: how many lines to display
     builder=: show only builds for this builder. Multiple builder= arguments
               can be used to see builds from any builder in the set.
+    reload=: reload the page after this many seconds
     """
 
     title = "Recent Builds"
@@ -89,7 +93,23 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
     def getChild(self, path, req):
         status = self.getStatus(req)
         builder = status.getBuilder(path)
-        return OneLinePerBuildOneBuilder(builder)
+        return OneLinePerBuildOneBuilder(builder, numbuilds=self.numbuilds)
+
+    def get_reload_time(self, request):
+        if "reload" in request.args:
+            try:
+                reload_time = int(request.args["reload"][0])
+                return max(reload_time, 15)
+            except ValueError:
+                pass
+        return None
+
+    def head(self, request):
+        head = ''
+        reload_time = self.get_reload_time(request)
+        if reload_time is not None:
+            head += '<meta http-equiv="refresh" content="%d">\n' % reload_time
+        return head
 
     def body(self, req):
         status = self.getStatus(req)
@@ -99,15 +119,17 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
         branches = [b for b in req.args.get("branch", []) if b]
 
         g = status.generateFinishedBuilds(builders, map_branches(branches),
-                                          numbuilds)
+                                          numbuilds, max_search=numbuilds)
 
         data = ""
 
         # really this is "up to %d builds"
+        html_branches = map(html.escape, branches)
         data += "<h1>Last %d finished builds: %s</h1>\n" % \
-                (numbuilds, ", ".join(branches))
+                (numbuilds, ", ".join(html_branches))
         if builders:
-            data += ("<p>of builders: %s</p>\n" % (", ".join(builders)))
+            html_builders = map(html.escape, builders)
+            data += ("<p>of builders: %s</p>\n" % (", ".join(html_builders)))
         data += "<ul>\n"
         got = 0
         building = False
@@ -128,10 +150,12 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
         if control is not None:
             if building:
                 stopURL = "builders/_all/stop"
-                data += make_stop_form(stopURL, True, "Builds")
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req),
+                                       True, "Builds")
             if online:
                 forceURL = "builders/_all/force"
-                data += make_force_build_form(forceURL, True)
+                data += make_force_build_form(forceURL,
+                                              self.isUsingUserPasswd(req), True)
 
         return data
 
@@ -158,8 +182,9 @@ class OneLinePerBuildOneBuilder(HtmlResource, OneLineMixin):
                                                 numbuilds)
 
         data = ""
+        html_branches = map(html.escape, branches)
         data += ("<h1>Last %d builds of builder %s: %s</h1>\n" %
-                 (numbuilds, self.builder_name, ", ".join(branches)))
+                 (numbuilds, self.builder_name, ", ".join(html_branches)))
         data += "<ul>\n"
         got = 0
         for build in g:
@@ -194,7 +219,8 @@ class OneBoxPerBuilder(HtmlResource):
 
         data = ""
 
-        data += "<h2>Latest builds: %s</h2>\n" % ", ".join(branches)
+        html_branches = map(html.escape, branches)
+        data += "<h2>Latest builds: %s</h2>\n" % ", ".join(html_branches)
         data += "<table>\n"
 
         building = False
@@ -239,10 +265,12 @@ class OneBoxPerBuilder(HtmlResource):
         if control is not None:
             if building:
                 stopURL = "builders/_all/stop"
-                data += make_stop_form(stopURL, True, "Builds")
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req),
+                                       True, "Builds")
             if online:
                 forceURL = "builders/_all/force"
-                data += make_force_build_form(forceURL, True)
+                data += make_force_build_form(forceURL,
+                                              self.isUsingUserPasswd(req), True)
 
         return data
 
@@ -294,8 +322,10 @@ class WebStatus(service.MultiService):
              query-arguments used by 'waterfall' can be added to
              influence the feed output.
      /grid : another summary display that shows a grid of builds, with
-             sourcestamps on the x axis, and builders on the y.  Query 
+             sourcestamps on the x axis, and builders on the y.  Query
              arguments similar to those for the waterfall can be added.
+     /tgrid : similar to the grid display, but the commits are down the
+              left side, and the build hosts are across the top.
      /builders/BUILDERNAME: a page summarizing the builder. This includes
                             references to the Schedulers that feed it,
                             any builds currently in the queue, which
@@ -368,7 +398,9 @@ class WebStatus(service.MultiService):
     # all the changes).
 
     def __init__(self, http_port=None, distrib_port=None, allowForce=False,
-                       public_html="public_html", site=None):
+                 public_html="public_html", site=None, numbuilds=20,
+                 num_events=200, num_events_max=None, auth=None,
+                 order_console_by_time=False):
         """Run a web server that provides Buildbot status.
 
         @type  http_port: int or L{twisted.application.strports} string
@@ -412,6 +444,31 @@ class WebStatus(service.MultiService):
         @type site: None or L{twisted.web.server.Site}
         @param site: Use this if you want to define your own object instead of
                      using the default.`
+
+        @type numbuilds: int
+        @param numbuilds: Default number of entries in lists at the /one_line_per_build
+        and /builders/FOO URLs.  This default can be overriden both programatically ---
+        by passing the equally named argument to constructors of OneLinePerBuildOneBuilder
+        and OneLinePerBuild --- and via the UI, by tacking ?numbuilds=xy onto the URL.
+
+        @type num_events: int
+        @param num_events: Defaualt number of events to show in the waterfall.
+
+        @type num_events_max: int
+        @param num_events_max: The maximum number of events that are allowed to be
+        shown in the waterfall.  The default value of C{None} will disable this
+        check
+
+        @type auth: a L{status.web.auth.IAuth} or C{None}
+        @param auth: an object that performs authentication to restrict access
+                     to the C{allowForce} features. Ignored if C{allowForce}
+                     is not C{True}. If C{auth} is C{None}, people can force or
+                     stop builds without auth.
+
+        @type order_console_by_time: bool
+        @param order_console_by_time: Whether to order changes (commits) in the console
+                     view according to the time they were created (for VCS like Git) or
+                     according to their integer revision numbers (for VCS like SVN).
         """
 
         service.MultiService.__init__(self)
@@ -425,8 +482,23 @@ class WebStatus(service.MultiService):
                 distrib_port = "unix:%s" % distrib_port
         self.distrib_port = distrib_port
         self.allowForce = allowForce
+        self.num_events = num_events
+        if num_events_max:
+            assert num_events_max >= num_events
+            self.num_events_max = num_events_max
         self.public_html = public_html
 
+        if self.allowForce and auth:
+            assert IAuth.providedBy(auth)
+            self.auth = auth
+        else:
+            if auth:
+                log.msg("Warning: Ignoring authentication. allowForce must be"
+                        " set to True use this")
+            self.auth = None
+
+        self.orderConsoleByTime = order_console_by_time
+
         # If we were given a site object, go ahead and use it.
         if site:
             self.site = site
@@ -437,7 +509,8 @@ class WebStatus(service.MultiService):
             self.site = server.Site(root)
         self.childrenToBeAdded = {}
 
-        self.setupUsualPages()
+        self.setupUsualPages(numbuilds=numbuilds, num_events=num_events,
+                             num_events_max=num_events_max)
 
         # the following items are accessed by HtmlResource when it renders
         # each page.
@@ -460,18 +533,25 @@ class WebStatus(service.MultiService):
             s = strports.service(self.distrib_port, f)
             s.setServiceParent(self)
 
-    def setupUsualPages(self):
+    def setupUsualPages(self, numbuilds, num_events, num_events_max):
         #self.putChild("", IndexOrWaterfallRedirection())
-        self.putChild("waterfall", WaterfallStatusResource())
+        self.putChild("waterfall", WaterfallStatusResource(num_events=num_events,
+                                        num_events_max=num_events_max))
         self.putChild("grid", GridStatusResource())
+        self.putChild("console", ConsoleStatusResource(
+                orderByTime=self.orderConsoleByTime))
+        self.putChild("tgrid", TransposedGridStatusResource())
         self.putChild("builders", BuildersResource()) # has builds/steps/logs
         self.putChild("changes", ChangesResource())
         self.putChild("buildslaves", BuildSlavesResource())
+        self.putChild("buildstatus", BuildStatusStatusResource())
         #self.putChild("schedulers", SchedulersResource())
-        self.putChild("one_line_per_build", OneLinePerBuild())
+        self.putChild("one_line_per_build",
+                      OneLinePerBuild(numbuilds=numbuilds))
         self.putChild("one_box_per_builder", OneBoxPerBuilder())
         self.putChild("xmlrpc", XMLRPCServer())
         self.putChild("about", AboutBuildbot())
+        self.putChild("authfail", AuthFailResource())
 
     def __repr__(self):
         if self.http_port is None:
@@ -545,11 +625,30 @@ class WebStatus(service.MultiService):
 
     def getChangeSvc(self):
         return self.master.change_svc
+
     def getPortnum(self):
         # this is for the benefit of unit tests
         s = list(self)[0]
         return s._port.getHost().port
 
+    def isUsingUserPasswd(self):
+        """Returns boolean to indicate if this WebStatus uses authentication"""
+        if self.auth:
+            return True
+        return False
+
+    def authUser(self, user, passwd):
+        """Check that user/passwd is a valid user/pass tuple and can should be
+        allowed to perform the action. If this WebStatus is not password
+        protected, this function returns False."""
+        if not self.isUsingUserPasswd():
+            return False
+        if self.auth.authenticate(user, passwd):
+            return True
+        log.msg("Authentication failed for '%s': %s" % (user,
+                                                        self.auth.errmsg()))
+        return False
+
 # resources can get access to the IStatus by calling
 # request.site.buildbot_service.getStatus()
 
@@ -602,7 +701,7 @@ class Waterfall(WebStatus):
                 pass
             else:
                 data = open(css, "rb").read()
-                self.putChild("buildbot.css", static.Data(data, "text/plain"))
+                self.putChild("buildbot.css", static.Data(data, "text/css"))
         self.favicon = favicon
         self.robots_txt = robots_txt
         if favicon:
