@@ -6,7 +6,9 @@ from twisted.internet import defer, reactor
 import urllib, time
 from twisted.python import log
 from buildbot.status.web.base import HtmlResource, make_row, make_stop_form, \
-     css_classes, path_to_builder
+     make_extra_property_row, css_classes, path_to_builder, path_to_slave, \
+     make_name_user_passwd_form, getAndCheckProperties
+
 
 from buildbot.status.web.tests import TestsResource
 from buildbot.status.web.step import StepsResource
@@ -30,11 +32,10 @@ class StatusResourceBuild(HtmlResource):
     def body(self, req):
         b = self.build_status
         status = self.getStatus(req)
-        projectURL = status.getProjectURL()
         projectName = status.getProjectName()
+        projectURL = status.getProjectURL()
         data = ('<div class="title"><a href="%s">%s</a></div>\n'
                 % (self.path_to_root(req), projectName))
-        # the color in the following line gives python-mode trouble
         builder_name = b.getBuilder().getName()
         data += ("<h1><a href=\"%s\">Builder %s</a>: Build #%d</h1>\n"
                  % (path_to_builder(req, b.getBuilder()),
@@ -50,9 +51,10 @@ class StatusResourceBuild(HtmlResource):
 
             if self.build_control is not None:
                 stopURL = urllib.quote(req.childLink("stop"))
-                data += make_stop_form(stopURL)
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req))
 
         if b.isFinished():
+            # Results map loosely to css_classes
             results = b.getResults()
             data += "<h2>Results:</h2>\n"
             text = " ".join(b.getText())
@@ -90,7 +92,11 @@ class StatusResourceBuild(HtmlResource):
 
         # TODO: turn this into a table, or some other sort of definition-list
         # that doesn't take up quite so much vertical space
-        data += "<h2>Buildslave:</h2>\n %s\n" % html.escape(b.getSlavename())
+        try:
+            slaveurl = path_to_slave(req, status.getSlave(b.getSlavename()))
+            data += "<h2>Buildslave:</h2>\n <a href=\"%s\">%s</a>\n" % (html.escape(slaveurl), html.escape(b.getSlavename()))
+        except KeyError:
+            data += "<h2>Buildslave:</h2>\n %s\n" % html.escape(b.getSlavename())
         data += "<h2>Reason:</h2>\n%s\n" % html.escape(b.getReason())
 
         data += "<h2>Steps and Logfiles:</h2>\n"
@@ -100,26 +106,44 @@ class StatusResourceBuild(HtmlResource):
 #        for name, target in urls.items():
 #            text.append('[<a href="%s" class="%s">%s</a>]' %
 #                        (target, ex_url_class, html.escape(name)))
-        if b.getLogs():
-            data += "<ol>\n"
-            for s in b.getSteps():
-                name = s.getName()
-                data += (" <li><a href=\"%s\">%s</a> [%s]\n"
-                         % (req.childLink("steps/%s" % urllib.quote(name)),
-                            name,
-                            " ".join(s.getText())))
-                if s.getLogs():
-                    data += "  <ol>\n"
-                    for logfile in s.getLogs():
-                        logname = logfile.getName()
-                        logurl = req.childLink("steps/%s/logs/%s" %
-                                               (urllib.quote(name),
-                                                urllib.quote(logname)))
-                        data += ("   <li><a href=\"%s\">%s</a></li>\n" %
-                                 (logurl, logfile.getName()))
-                    data += "  </ol>\n"
-                data += " </li>\n"
+        data += "<ol>\n"
+        for s in b.getSteps():
+            name = s.getName()
+            time_to_run = 0
+            (start, end) = s.getTimes()
+            if start and end:
+              time_to_run = end - start
+            if s.isFinished():
+                css_class = css_classes[s.getResults()[0]]
+            elif s.isStarted():
+                css_class = "running"
+            else:
+                css_class = ""
+            data += (' <li><span class="%s"><a href=\"%s\">%s</a> [%s] [%d seconds]</span>\n'
+                     % (css_class, 
+                        req.childLink("steps/%s" % urllib.quote(name)),
+                        name,
+                        " ".join(s.getText()),
+                        time_to_run))
+            data += "  <ol>\n"
+            if s.getLogs():
+                for logfile in s.getLogs():
+                    logname = logfile.getName()
+                    logurl = req.childLink("steps/%s/logs/%s" %
+                                           (urllib.quote(name),
+                                            urllib.quote(logname)))
+                    data += ("   <li><a href=\"%s\">%s</a></li>\n" %
+                             (logurl, logfile.getName()))
+            if s.getURLs():
+                for url in s.getURLs().items():
+                    logname = url[0]
+                    logurl = url[1]
+                    data += ('   <li><a href="%s">%s</a></li>\n' %
+                             (logurl, html.escape(logname)))
             data += "</ol>\n"
+            data += " </li>\n"
+
+        data += "</ol>\n"
 
         data += "<h2>Build Properties:</h2>\n"
         data += "<table><tr><th valign=\"left\">Name</th><th valign=\"left\">Value</th><th valign=\"left\">Source</th></tr>\n"
@@ -143,6 +167,19 @@ class StatusResourceBuild(HtmlResource):
         else:
             data += "<div>no responsible users</div>\n"
 
+
+        (start, end) = b.getTimes()
+        data += "<h2>Timing</h2>\n"
+        data += "<table>\n"
+        data += "<tr><td>Start</td><td>%s</td></tr>\n" % time.ctime(start)
+        if end:
+            data += "<tr><td>End</td><td>%s</td></tr>\n" % time.ctime(end)
+            data += "<tr><td>Elapsed</td><td>%s</td></tr>\n" % util.formatInterval(end - start)
+        else:
+            now = util.now()
+            data += "<tr><td>Elapsed</td><td>%s</td></tr>\n" % util.formatInterval(now - start)
+        data += "</table>\n"
+
         if ss.changes:
             data += "<h2>All Changes</h2>\n"
             data += "<ol>\n"
@@ -168,50 +205,36 @@ class StatusResourceBuild(HtmlResource):
                          "after this build was started <b>will</b> be \n"
                          "included in a rebuild.</p>\n")
             rebuildURL = urllib.quote(req.childLink("rebuild"))
-            data += ('<form action="%s" class="command rebuild">\n'
+            data += ('<form method="post" action="%s" class="command rebuild">\n'
                      % rebuildURL)
-            data += make_row("Your name:",
-                             "<input type='text' name='username' />")
+            data += make_name_user_passwd_form(self.isUsingUserPasswd(req))
+            data += make_extra_property_row(1)
+            data += make_extra_property_row(2)
+            data += make_extra_property_row(3)
             data += make_row("Reason for re-running build:",
                              "<input type='text' name='comments' />")
             data += '<input type="submit" value="Rebuild" />\n'
             data += '</form>\n'
 
-        # TODO: this stuff should be generated by a template of some sort
-        data += '<hr /><div class="footer">\n'
-
-        welcomeurl = self.path_to_root(req) + "index.html"
-        data += '[<a href="%s">welcome</a>]\n' % welcomeurl
-        data += "<br />\n"
-
-        data += '<a href="http://buildbot.sourceforge.net/">Buildbot</a>'
-        data += "-%s " % version
-        if projectName:
-            data += "working for the "
-            if projectURL:
-                data += "<a href=\"%s\">%s</a> project." % (projectURL,
-                                                            projectName)
-            else:
-                data += "%s project." % projectName
-        data += "<br />\n"
-        data += ("Page built: " +
-                 time.strftime("%a %d %b %Y %H:%M:%S",
-                               time.localtime(util.now()))
-                 + "\n")
-        data += '</div>\n'
+        data += self.footer(status, req)
 
         return data
 
     def stop(self, req):
+        if self.isUsingUserPasswd(req):
+            if not self.authUser(req):
+                return Redirect("../../../authfailed")
         b = self.build_status
         c = self.build_control
         log.msg("web stopBuild of build %s:%s" % \
                 (b.getBuilder().getName(), b.getNumber()))
         name = req.args.get("username", ["<unknown>"])[0]
         comments = req.args.get("comments", ["<no reason specified>"])[0]
+        # html-quote both the username and comments, just to be safe
         reason = ("The web-page 'stop build' button was pressed by "
-                  "'%s': %s\n" % (name, comments))
-        c.stopBuild(reason)
+                  "'%s': %s\n" % (html.escape(name), html.escape(comments)))
+        if c:
+            c.stopBuild(reason)
         # we're at http://localhost:8080/svn-hello/builds/5/stop?[args] and
         # we want to go to: http://localhost:8080/svn-hello
         r = Redirect("../..")
@@ -220,6 +243,9 @@ class StatusResourceBuild(HtmlResource):
         return DeferredResource(d)
 
     def rebuild(self, req):
+        if self.isUsingUserPasswd(req):
+            if not self.authUser(req):
+                return Redirect("../../../authfailed")
         b = self.build_status
         bc = self.builder_control
         builder_name = b.getBuilder().getName()
@@ -227,13 +253,14 @@ class StatusResourceBuild(HtmlResource):
         name = req.args.get("username", ["<unknown>"])[0]
         comments = req.args.get("comments", ["<no reason specified>"])[0]
         reason = ("The web-page 'rebuild' button was pressed by "
-                  "'%s': %s\n" % (name, comments))
-        if not bc or not b.isFinished():
+                  "'%s': %s\n" % (html.escape(name), html.escape(comments)))
+        extraProperties = getAndCheckProperties(req)
+        if not bc or not b.isFinished() or extraProperties is None:
             log.msg("could not rebuild: bc=%s, isFinished=%s"
                     % (bc, b.isFinished()))
             # TODO: indicate an error
         else:
-            bc.resubmitBuild(b, reason)
+            bc.resubmitBuild(b, reason, extraProperties)
         # we're at
         # http://localhost:8080/builders/NAME/builds/5/rebuild?[args]
         # Where should we send them?
