@@ -2,7 +2,7 @@
 
 from twisted.python import log
 
-from buildbot.status import tests, builder
+from buildbot.status import builder
 from buildbot.status.builder import SUCCESS, FAILURE, WARNINGS, SKIPPED
 from buildbot.process.buildstep import LogLineObserver, OutputProgressObserver
 from buildbot.process.buildstep import RemoteShellCommand
@@ -34,6 +34,7 @@ class HLint(ShellCommand):
 
     def __init__(self, python=None, **kwargs):
         ShellCommand.__init__(self, **kwargs)
+        self.addFactoryArguments(python=python)
         self.python = python
 
     def start(self):
@@ -134,7 +135,7 @@ def countFailedTests(output):
 
 
 class TrialTestCaseCounter(LogLineObserver):
-    _line_re = re.compile(r'^([\w\.]+) \.\.\. \[([^\]]+)\]$')
+    _line_re = re.compile(r'^(?:Doctest: )?([\w\.]+) \.\.\. \[([^\]]+)\]$')
     numTests = 0
     finished = False
 
@@ -145,6 +146,8 @@ class TrialTestCaseCounter(LogLineObserver):
         #  2.1.0: buildbot.test.test_runner.Create.testSlave ... [OK]
         #  2.4.0: buildbot.test.test_runner.Create.testSlave ... [OK]
         # Let's just handle the most recent version, since it's the easiest.
+        # Note that doctests create lines line this:
+        #  Doctest: viff.field.GF ... [OK]
 
         if self.finished:
             return
@@ -183,12 +186,6 @@ class Trial(ShellCommand):
     do the importing and running. The 'tests' parameter controls which tests
     trial will run: it can be a string or a list of strings.
 
-    You can also use a higher-level module name and pass the --recursive flag
-    to trial: this will search recursively within the named module to find
-    all test cases. For large multiple-test-directory projects like Twisted,
-    this means you can avoid specifying all the test directories explicitly.
-    Something like 'trial --recursive twisted' will pick up everything.
-
     To find these test cases, you must set a PYTHONPATH that allows something
     like 'import petmail.test' to work. For packages that don't use a
     separate top-level 'lib' directory, PYTHONPATH=. will work, and will use
@@ -332,6 +329,17 @@ class Trial(ShellCommand):
                        timeout.
         """
         ShellCommand.__init__(self, **kwargs)
+        self.addFactoryArguments(reactor=reactor,
+                                 python=python,
+                                 trial=trial,
+                                 testpath=testpath,
+                                 tests=tests,
+                                 testChanges=testChanges,
+                                 recurse=recurse,
+                                 randomly=randomly,
+                                 trialMode=trialMode,
+                                 trialArgs=trialArgs,
+                                 )
 
         if python:
             self.python = python
@@ -675,12 +683,6 @@ class ProcessDocs(ShellCommand):
         @type    workdir: string
         @keyword workdir: the workdir to start from: must be the base of the
                           Twisted tree
-
-        @type    results: triple of (int, int, string)
-        @keyword results: [rc, warnings, output]
-                          - rc==0 if all files were converted successfully.
-                          - warnings is a count of hlint warnings. 
-                          - output is the verbose output of the command.
         """
         ShellCommand.__init__(self, **kwargs)
 
@@ -744,10 +746,6 @@ class BuildDebs(ShellCommand):
         @type    workdir: string
         @keyword workdir: the workdir to start from (must be the base of the
                           Twisted tree)
-        @type    results: double of [int, string]
-        @keyword results: [rc, output].
-                          - rc == 0 if all .debs were created successfully
-                          - output: string with any errors or warnings
         """
         ShellCommand.__init__(self, **kwargs)
 
