@@ -7,6 +7,10 @@ from buildbot import master
 from buildbot.changes import pb
 from buildbot.scripts import runner
 
+import sys
+if sys.version_info[:3] < (2,4,0):
+    from sets import Set as set
+
 d1 = {'files': ["Project/foo.c", "Project/bar/boo.c"],
       'who': "marvin",
       'comments': "Some changes in Project"}
@@ -19,6 +23,10 @@ d3 = {'files': ["Project/baz.c", "OtherProject/bloo.c"],
 d4 = {'files': ["trunk/baz.c", "branches/foobranch/foo.c", "trunk/bar.c"],
       'who': "alice",
       'comments': "mixed changes"}
+d5 = {'files': ["Project/foo.c"],
+      'who': "trillian",
+      'comments': "Some changes in Project",
+      'category': "categoryA"}
 
 class TestChangePerspective(unittest.TestCase):
 
@@ -33,8 +41,8 @@ class TestChangePerspective(unittest.TestCase):
         p.perspective_addChange(d1)
         self.failUnlessEqual(len(self.changes), 1)
         c1 = self.changes[0]
-        self.failUnlessEqual(c1.files,
-                             ["Project/foo.c", "Project/bar/boo.c"])
+        self.failUnlessEqual(set(c1.files),
+                             set(["Project/foo.c", "Project/bar/boo.c"]))
         self.failUnlessEqual(c1.comments, "Some changes in Project")
         self.failUnlessEqual(c1.who, "marvin")
 
@@ -44,7 +52,7 @@ class TestChangePerspective(unittest.TestCase):
         p.perspective_addChange(d1)
         self.failUnlessEqual(len(self.changes), 1)
         c1 = self.changes[-1]
-        self.failUnlessEqual(c1.files, ["foo.c", "bar/boo.c"])
+        self.failUnlessEqual(set(c1.files), set(["foo.c", "bar/boo.c"]))
         self.failUnlessEqual(c1.comments, "Some changes in Project")
         self.failUnlessEqual(c1.who, "marvin")
 
@@ -55,7 +63,7 @@ class TestChangePerspective(unittest.TestCase):
         self.failUnlessEqual(len(self.changes), 2)
 
         c3 = self.changes[-1]
-        self.failUnlessEqual(c3.files, ["baz.c"])
+        self.failUnlessEqual(set(c3.files), set(["baz.c"]))
         self.failUnlessEqual(c3.comments, "mixed changes")
         self.failUnlessEqual(c3.who, "alice")
 
@@ -65,7 +73,7 @@ class TestChangePerspective(unittest.TestCase):
         p.perspective_addChange(d1)
         self.failUnlessEqual(len(self.changes), 1)
         c1 = self.changes[-1]
-        self.failUnlessEqual(c1.files, ["boo.c"])
+        self.failUnlessEqual(set(c1.files), set(["boo.c"]))
         self.failUnlessEqual(c1.comments, "Some changes in Project")
         self.failUnlessEqual(c1.who, "marvin")
 
@@ -81,7 +89,7 @@ class TestChangePerspective(unittest.TestCase):
         p.perspective_addChange(d4)
         self.failUnlessEqual(len(self.changes), 1)
         c1 = self.changes[-1]
-        self.failUnlessEqual(c1.files, ["baz.c", "bar.c"])
+        self.failUnlessEqual(set(c1.files), set(["baz.c", "bar.c"]))
         self.failUnlessEqual(c1.comments, "mixed changes")
 
     def testPrefix4(self):
@@ -90,10 +98,15 @@ class TestChangePerspective(unittest.TestCase):
         p.perspective_addChange(d4)
         self.failUnlessEqual(len(self.changes), 1)
         c1 = self.changes[-1]
-        self.failUnlessEqual(c1.files, ["foo.c"])
+        self.failUnlessEqual(set(c1.files), set(["foo.c"]))
         self.failUnlessEqual(c1.comments, "mixed changes")
 
-
+    def testCategory(self):
+        p = pb.ChangePerspective(self, None)
+        p.perspective_addChange(d5)
+        self.failUnlessEqual(len(self.changes), 1)
+        c1 = self.changes[0]
+        self.failUnlessEqual(c1.category, "categoryA")
 
 config_empty = """
 BuildmasterConfig = c = {}
@@ -144,6 +157,7 @@ class Sender(unittest.TestCase):
         self.options = {'username': "alice",
                         'master': "localhost:%d" % port,
                         'files': ["foo.c"],
+                        'category': "categoryA",
                         }
 
         d = runner.sendchange(self.options)
@@ -158,6 +172,7 @@ class Sender(unittest.TestCase):
         self.failUnlessEqual(c.files, ["foo.c"])
         self.failUnlessEqual(c.comments, "")
         self.failUnlessEqual(c.revision, None)
+        self.failUnlessEqual(c.category, "categoryA")
 
         self.options['revision'] = "r123"
         self.options['comments'] = "test change"
@@ -173,6 +188,7 @@ class Sender(unittest.TestCase):
         self.failUnlessEqual(c.files, ["foo.c"])
         self.failUnlessEqual(c.comments, "test change")
         self.failUnlessEqual(c.revision, "r123")
+        self.failUnlessEqual(c.category, "categoryA")
 
         # test options['logfile'] by creating a temporary file
         logfile = self.mktemp()
@@ -193,6 +209,7 @@ class Sender(unittest.TestCase):
         self.failUnlessEqual(c.files, ["foo.c"])
         self.failUnlessEqual(c.comments, "longer test change")
         self.failUnlessEqual(c.revision, "r123")
+        self.failUnlessEqual(c.category, "categoryA")
 
         # make sure that numeric revisions work too
         self.options['logfile'] = None
@@ -210,6 +227,7 @@ class Sender(unittest.TestCase):
         self.failUnlessEqual(c.files, ["foo.c"])
         self.failUnlessEqual(c.comments, "")
         self.failUnlessEqual(c.revision, 42)
+        self.failUnlessEqual(c.category, "categoryA")
 
         # verify --branch too
         self.options['branch'] = "branches/test"
@@ -226,3 +244,4 @@ class Sender(unittest.TestCase):
         self.failUnlessEqual(c.comments, "")
         self.failUnlessEqual(c.revision, 42)
         self.failUnlessEqual(c.branch, "branches/test")
+        self.failUnlessEqual(c.category, "categoryA")
