@@ -5,19 +5,19 @@ from twisted.trial import unittest
 from twisted.internet import reactor, defer
 
 from buildbot.test.runutils import RunMixin
-from buildbot.twcompat import maybeWait
 from buildbot.status import base
 
 config_1 = """
 from buildbot import scheduler
 from buildbot.process import factory
 from buildbot.steps import dummy
+from buildbot.buildslave import BuildSlave
 s = factory.s
 from buildbot.test.test_locks import LockStep
+from buildbot.config import BuilderConfig
 
 BuildmasterConfig = c = {}
-c['bots'] = [('bot1', 'sekrit'), ('bot2', 'sekrit')]
-c['sources'] = []
+c['slaves'] = [BuildSlave('bot1', 'sekrit'), BuildSlave('bot2', 'sekrit')]
 c['schedulers'] = []
 c['slavePortnum'] = 0
 
@@ -39,8 +39,10 @@ f_slowpass = factory.BuildFactory([s(dummy.Dummy, timeout=2)])
 f_fastfail = factory.BuildFactory([s(dummy.FailingDummy, timeout=1)])
 
 def builder(name, f):
-    d = {'name': name, 'slavename': 'bot1', 'builddir': name, 'factory': f}
-    return d
+    return BuilderConfig(
+        name=name,
+        slavename='bot1',
+        factory=f)
 
 c['builders'] = [builder('slowpass', f_slowpass),
                  builder('fastfail', f_fastfail),
@@ -68,7 +70,7 @@ class Dependencies(RunMixin, unittest.TestCase):
         self.master.startService()
         d = self.connectSlave(["slowpass", "fastfail", "fastpass",
                                "b3", "b4", "b5"])
-        return maybeWait(d)
+        return d
 
     def findScheduler(self, name):
         for s in self.master.allSchedulers():
@@ -96,7 +98,7 @@ class Dependencies(RunMixin, unittest.TestCase):
         d = defer.Deferred()
         d.addCallback(self._testRun_Fail_1)
         reactor.callLater(5, d.callback, None)
-        return maybeWait(d)
+        return d
 
     def _testRun_Fail_1(self, res):
         # 'slowpass' and 'fastfail' should have run one build each
@@ -138,7 +140,7 @@ class Dependencies(RunMixin, unittest.TestCase):
         d = defer.Deferred()
         d.addCallback(self._testRun_Pass_1)
         reactor.callLater(5, d.callback, None)
-        return maybeWait(d)
+        return d
 
     def _testRun_Pass_1(self, res):
         # 'fastpass' and 'slowpass' should have run one build each
